@@ -722,6 +722,8 @@ func (s *SourceState) ForEach(f func(RelPath, SourceStateEntry) error) error {
 
 // Ignore returns if targetRelPath should be ignored.
 func (s *SourceState) Ignore(targetRelPath RelPath) bool {
+	s.Lock()
+	defer s.Unlock()
 	return s.ignore.match(targetRelPath.String())
 }
 
@@ -800,7 +802,7 @@ func (s *SourceState) Read(ctx context.Context, options *ReadOptions) error {
 		allSourceStateEntries[relPath] = append(allSourceStateEntries[relPath], sourceStateEntries...)
 		allSourceStateEntriesMu.Unlock()
 	}
-	walkFunc := func(ctx context.Context, sourceAbsPath AbsPath, fileInfo fs.FileInfo, err error) error {
+	walkFunc := func(sourceAbsPath AbsPath, fileInfo fs.FileInfo, err error) error {
 		if err != nil {
 			return err
 		}
@@ -896,7 +898,7 @@ func (s *SourceState) Read(ctx context.Context, options *ReadOptions) error {
 			}
 		}
 	}
-	if err := concurrentWalkSourceDir(ctx, s.system, s.sourceDirAbsPath, walkFunc); err != nil {
+	if err := WalkSourceDir(s.system, s.sourceDirAbsPath, walkFunc); err != nil {
 		return err
 	}
 
@@ -1117,6 +1119,10 @@ func (s *SourceState) addPatterns(patternSet *patternSet, sourceAbsPath AbsPath,
 	if err != nil {
 		return err
 	}
+
+	s.Lock()
+	defer s.Unlock()
+
 	dir := sourceRelPath.Dir().TargetRelPath("")
 	scanner := bufio.NewScanner(bytes.NewReader(data))
 	lineNumber := 0
